var Text = GetSystemFont();
var sqrt = Math.sqrt;
var lp = new Vector();
var v1 = new Vector();
var v2 = new Vector();

function Spotlight( v, c, i )
{
	this.Vector = v;
	this.Color = c;
	this.Intensity = i;
	this.FallOff = 1;
}

Spotlight.prototype.LitColor = function( p, n, c )
{
	lp.x = p.tx - this.Vector.tx;
	lp.y = p.ty - this.Vector.ty;
	lp.z = p.tz - this.Vector.tz;

	var Mag;

	Mag = Math.sqrt(( lp.x*lp.x ) + ( lp.y*lp.y ) + ( lp.z*lp.z ));
	lp.x /= Mag;
	lp.y /= Mag;
	lp.z /= Mag;

	var result = (( n.x * lp.x ) + ( n.y * lp.y ) + ( n.z * lp.z )) * this.Intensity;

	if( result < 0 )
	{
		return c;
	}

	return BlendColorsWeighted( this.Color, c, result * this.FallOff, 1 - result);
}

function Pointlight( v, c, i )
{
	this.Vector = v;
	this.Color = c;
	this.Intensity = i;
	this.FallOff = 1;
}

Pointlight.prototype.LitColor = function( p, n, c )
{
	lp.x = p.tx - this.Vector.tx;
	lp.y = p.ty - this.Vector.ty;
	lp.z = p.tz - this.Vector.tz;
	
	result = ( this.FallOff - Math.sqrt(( lp.x*lp.x ) + ( lp.y*lp.y ) + ( lp.z*lp.z )) );

	if( result < 0 )
	{
		return c;
	}

	return BlendColorsWeighted( this.Color, c, result * this.Intensity, this.FallOff - result);
}

function Vector(x,y,z)
{
	this.x = x;
	this.y = y;
	this.z = z;
	this.tx = x;
	this.ty = y;
	this.tz = z;
}

Vector.prototype.Reset = function()
{
	with(this)
	{
		tx = x;
		ty = y;
		tz = z;
	}
	
	return this;
}

Vector.prototype.Rotate = function(P)
{
	var old;	

	if( P.x )
	{
		old = this.ty;
		this.ty = (Math.cos (P.x) * old) - (Math.sin (P.x) * this.tz); 
		this.tz = (Math.sin (P.x) * old) + (Math.cos (P.x) * this.tz);
	}
	if( P.y )
	{
		old = this.tz;
		this.tz = (Math.cos (P.y) * old) - (Math.sin (P.y) * this.tx);
		this.tx = (Math.cos (P.y) * this.tx) + (Math.sin (P.y) * old);
	}
	if( P.z )
	{
		old = this.tx;
		this.tx = (Math.cos (P.z) * old) - (Math.sin (P.z) * this.ty);
		this.ty = (Math.sin (P.z) * old) + (Math.cos (P.z) * this.ty);
	}
	return this;
}

Vector.prototype.Translate = function(P)
{
	this.tx += P.x;
	this.ty += P.y;
	this.tz += P.z;
	
	return this;
}

Vector.prototype.Scale = function(M)
{
	this.tx *= M.x;
	this.ty *= M.y;
	this.tz *= M.z;
}

Vector.prototype.Project = function(F)
{
	this.px = this.tx * F.z / this.tz + F.x;
	this.py = this.ty * F.z / this.tz + F.y;
}

function Face( p1, p2, p3, c )
{
	this.Vector = new Array(3);
	this.Normal = new Vector(0,0,0);
	this.Color = c;

	this.Vector[0] = p1;
	this.Vector[1] = p2;
	this.Vector[2] = p3;
}

Face.prototype.Normalize = function ()
{
	v1.x = this.Vector[2].tx - this.Vector[1].tx;
	v1.y = this.Vector[2].ty - this.Vector[1].ty;
	v1.z = this.Vector[2].tz - this.Vector[1].tz;
	v2.x = this.Vector[0].tx - this.Vector[1].tx;
	v2.y = this.Vector[0].ty - this.Vector[1].ty;
	v2.z = this.Vector[0].tz - this.Vector[1].tz;
	
	this.Normal.x = ( v1.y * v2.z ) - ( v1.z * v2.y );
	this.Normal.y = ( v1.x * v2.z ) - ( v1.z * v2.x );
	this.Normal.z = ( v1.y * v2.x ) - ( v1.x * v2.y );

	delete v1;
	delete v2;

	var Mag = Math.sqrt(	(this.Normal.x * this.Normal.x) + 
					(this.Normal.y * this.Normal.y) + 
					(this.Normal.z * this.Normal.z));	

	this.Normal.x /= Mag;
	this.Normal.y /= Mag;
	this.Normal.z /= Mag;
}

Face.prototype.Render = function(C)
{
	this.Normalize();

	if( this.Normal.Z > 0 )
	{
		Triangle(	this.Vector[0].px,this.Vector[0].py,
				this.Vector[1].px,this.Vector[1].py,
				this.Vector[2].px,this.Vector[2].py, C);
	}
}

function Model()
{
	this.Light = new Array();
	this.Face = new Array();
	this.Vector = new Array();
}

Model.prototype.Frustrum = function(F)
{
	this.Frustrum = F;
}

Model.prototype.AddFace = function(F)
{
	this.Face.push(F);
}

Model.prototype.AddLight = function(L)
{
	this.Light.push(L);
}

Model.prototype.AddVector = function(V)
{
	this.Vector.push(V);
}

Model.prototype.Scale = function(M)
{
	for( var F = 0; F < this.Vector.length; F++ )
	{
		this.Vector[F].Scale(M);
	}

	return this;
}

Model.prototype.Translate = function(M)
{
	for( var F = 0; F < this.Vector.length; F++ )
	{
		this.Vector[F].Translate(M);
	}

	return this;
}

Model.prototype.Rotate = function(M)
{
	for( var F = 0; F < this.Vector.length; F++ )
	{
		this.Vector[F].Rotate(M);
	}

	return this;
}

Model.prototype.Project = function(M)
{
	for( var F = 0; F < this.Vector.length; F++ )
	{
		this.Vector[F].Project(M);
	}

	return this;
}

Model.prototype.Reset = function(M)
{
	for( var F = 0; F < this.Vector.length; F++ )
	{
		this.Vector[F].Reset();
	}

	return this;
}

Model.prototype.Render = function()
{
	var F, L;

	for( F = 0; F < this.Face.length; F++ )
	{
		this.Face[F].Normalize();
		
		if( this.Face[F].Normal.z != 0 )
		{
			var Color = new Array(3);

			for( V = 0; V < 3; V++ )
			{
				Color[V] = this.Face[F].Color;
				for( L = 0; L < this.Light.length; L++ )
				{
					Color[V] = this.Light[L].LitColor( this.Face[F].Vector[V], this.Face[F].Normal, Color[V] );
				}
			}

			GradientTriangle(	this.Face[F].Vector[0].px,this.Face[F].Vector[0].py,
						this.Face[F].Vector[1].px,this.Face[F].Vector[1].py,
						this.Face[F].Vector[2].px,this.Face[F].Vector[2].py, 
						Color[0],Color[1],Color[2]);
		}
	}
}

Model.prototype.LoadModel = function( ModelFile )
{
	var File = OpenRawFile( ModelFile );
	var ReadBuffer = File.read(File.getSize());	
	var Command = new String("");
	var i;

	for( i=0; i< ReadBuffer.length ; i++ )
	{
		Command += String.fromCharCode(ReadBuffer[i]);
	}

	Command.toLowerCase();
	
	Tokens = Command.split("\n").join(" ").split("\r").join(" ").split("\t").join(" ").split("(").join(" ").split(")").join(" ").split(",").join(" ").split(" ");

	for( i = Tokens.length - 1; i >= 0 ; i-- )
	{
		if( Tokens[i] == "" )
		{
			Tokens.splice(i,1);
		}
	}

	var Color = new Array();
	var VectorNames = new Object();

	for( i=0; i < Tokens.length; )
	{	
		switch( Tokens[i] )
		{
		case "color":
			Color[Tokens[i+1]] = CreateColor(Tokens[i+2],Tokens[i+3],Tokens[i+4],50);
			i+=5;
			break ;
		case "vector":
			
			this.AddVector(VectorNames[Tokens[i+1]] = new Vector(Tokens[i+2],Tokens[i+3],Tokens[i+4]));
			i+=5;
			break ;
		case "poly":
			this.AddFace(new Face(	VectorNames[Tokens[i+4]],
								VectorNames[Tokens[i+6]],
								VectorNames[Tokens[i+5]],
								Color[Tokens[i+2]])		);
			i+=7;
			break ;
		default :			
			i++;
		}
	}

	return Command;
}

function CreatePlane( Obj, x1, y1, x2, y2, z, detail )
{
	var x,y,P = 0;
	
	var Points = new Array();

	for( y = 0; y < (detail+1); y++ )
	{
		for( x = 0; x < (detail+1); x++ )
		{
			var GridPoint = new Vector( (x2 - x1) / detail * x + x1, (y2 - y1) / detail * y + y1, z );
			Points.push(GridPoint);
			Obj.AddVector(GridPoint);
		}
	}

	var C = CreateColor(100,100,100);

	for( y = 0; y < detail; y++ )
	{
		for( x = 0; x < detail; x++ )
		{
			var GridFace = new Face(	Points[(x+1)+y*(detail+1)],
								Points[x+y*(detail+1)],
								Points[x+(y+1)*(detail+1)], C);

			Obj.AddFace(GridFace);

			      GridFace = new Face(	Points[(x+1)+y*(detail+1)],
								Points[x+(y+1)*(detail+1)], 
								Points[(x+1)+(y+1)*(detail+1)], C);
			Obj.AddFace(GridFace);

		}
	}
}

Model.prototype.Sort = function()
{
	var max = Math.max;
	
	var FaceSortFunct = function ( a, b )
	{
		return	max( b.Vector[0].tz, b.Vector[1].tz, b.Vector[2].tz ) -
				max( a.Vector[0].tz, a.Vector[1].tz, a.Vector[2].tz );
	}

	this.Face.sort(FaceSortFunct);
}

Model.prototype.Merge = function(O)
{
	for( var i = 0; i < O.Vector.length; i++ )
	{
		with( O.Vector[i] )
		{
			x = tx;
			y = ty;
			z = tz;
		}
	}
	
	this.Vector = this.Vector.concat(O.Vector);
	this.Face = this.Face.concat(O.Face);
}

function game()
{
	var Rotation = new Vector(0,0,0);
	var Translation = new Vector(0,0,3);
	var Frustrum = new Vector(320,240,256);
	var LightPoint1 = new Vector(0,0,2);
	var Light1 = new Pointlight( LightPoint1, CreateColor(100,100,255), 0.5 );
	Light1.FallOff = 5;
	var LightPoint2 = new Vector(0,0,0);
	var Light2 = new Spotlight( LightPoint2, CreateColor(255,255,255), 1);
	Light2.FallOff = 0.1;

	var Cube = new Model();
	
	Cube.AddLight( Light1 );
	Cube.AddLight( Light2 );

//	CreatePlane(Cube,-1,-1,1,1,0,3);
	
	Cube.LoadModel("station.mdl");	

//	Cube.LoadModel("Plus3d.mdl");

//	var Plus = new Model();
//	Plus.LoadModel("Plus3d.mdl");
//	Plus.Scale( new Vector(.5,.5,.5) );
//	Plus.Translate( new Vector(0,0,5) );

//	Cube.Merge(Plus);	

	var Time = GetTime(), Frames = 0;

	while(!IsKeyPressed(KEY_Q))
	{
//		Rotation.x += Math.PI / 360;
		Rotation.y += Math.PI / 360;
//		Rotation.z += Math.PI / 360;
		
		Cube.Reset().Rotate(Rotation).Translate(Translation).Rotate(Rotation).Translate(Translation).Translate(Translation).Project(Frustrum);
		Cube.Sort();
		Cube.Render();

		Text.drawText(0,0,(Frames++)*1000/(GetTime()-Time));

		FlipScreen();
	}
}